@@ -1,4 +1,4 @@
-.densParzenWindow.x <- function(x, hx)
+.densParzenWindow.x <- function(x, hx, npts)
 {
   output <- .C("RdensParzenWindowX",
     n = as.integer(length(x)),
@@ -11,12 +11,21 @@
   if (output$error == 1) {
     stop("in densParzenWindow.x!", call. = FALSE); return(NA)
   }
+  
+  i <- !duplicated(output$x) 
 
-  i <- order(output$y)
-
-  output$x <- output$x[i]
+  output$x <- output$x[i] 
   output$y <- output$y[i]
-
+  
+  n <- length(output$y)
+  
+  if (n > npts) {
+    i <- sample.int(n, npts, replace = FALSE, prob = NULL)  
+  
+    output$x <- output$x[i]
+    output$y <- output$y[i]
+  }
+  
   rm(list = ls()[!(ls() %in% c("output"))])
 
   return(output)
