@@ -38,7 +38,7 @@ typedef enum {
     icPC,     // PC - Partition coefficient Bezdek (1981).
     icICLBIC, // ICL-BIC - Integrated classification likelihood criterion Biernacki et al.(1998).
     icD,      // D - Total of positive relative deviations Nagode & Fajdiga (2011).
-    icSSE,    // SSE - Sum of squares error Bishop (1998).
+    icSSE     // SSE - Sum of squares error Bishop (1998).
 } InformationCriterionType_e;
 
 typedef struct roughparametertype {
@@ -65,7 +65,22 @@ typedef struct additinalparametertype {
     int d;       // Golden section constant.
 } AdditionalParameterType;
 
-class Rebmix {
+class CompnentDistribution : public Base {
+public:
+    // Members.
+    Base                   *owner_;  // Owner object. 
+    ParametricFamilyType_e *pdf_;    // Parametric family types.
+    FLOAT                  **Theta_; // Component parameters.
+    // Constructor.
+    CompnentDistribution(Base *owner);
+    // Destructor.
+    ~CompnentDistribution();
+    // Methods.
+    int Realloc(int length_pdf, int length_Theta, int *length_theta);
+    int Memmove(CompnentDistribution *CmpTheta);
+}; // CompnentDistribution
+
+class Rebmix : public Base {
     // Methods.
     int Golden();
     int GlobalModeKNN(int *m, FLOAT **Y);
@@ -82,14 +97,11 @@ public:
     int                        o_;             // Number of paths.
     char                       **open_;        // Paths to open data files.
     char                       *save_;         // Path to the save data file.
-    int                        d_;             // Number of independent random variables.
     PreprocessingType_e        Preprocessing_; // Preprocessing type.
     int                        cmax_;          // Maximum number of components.
     InformationCriterionType_e Criterion_;     // Infromation criterion type.
     VariablesType_e            *Variables_;    // Types of variables.
-    ParametricFamilyType_e     *pdf_;          // Parametric family types.
-    FLOAT                      *Theta1_;       // Initial component parameters.
-    FLOAT                      *Theta2_;       // Initial component parameters.
+    CompnentDistribution       *IniTheta_;     // Initial component parameters.
     int                        length_K_;      // Length of K_.
     int                        *K_;            // Numbers of bins v or numbers of nearest neighbours k.
     FLOAT                      *y0_;           // Origins.
@@ -103,7 +115,7 @@ public:
     FLOAT                      **Y_;           // Dataset.
     // Output members.
     FLOAT                      *W_;            // Component weights.
-    ComponentDistributionType  *Theta_;        // Component parameters.
+    CompnentDistribution       **MixTheta_;    // Mixture parameters.
     SummaryParameterType       summary_;       // Summary.
     int                        opt_length_;    // Length of opt_c_, opt_IC_, opt_logL_ and opt_D_.
     int                        *opt_c_;        // Numbers of components for optimal v or for optimal k.
@@ -122,22 +134,22 @@ public:
     int PreprocessingKNN(int k, FLOAT *h, FLOAT **Y);
     int PreprocessingPW(FLOAT *h, FLOAT **Y);
     int PreprocessingH(FLOAT *h, FLOAT *y0, int *k, FLOAT **Y);
-    virtual int RoughEstimationKNN(FLOAT **Y, int k, FLOAT *h, FLOAT nl, int m, ComponentDistributionType *RigidTheta, ComponentDistributionType *LooseTheta);
-    virtual int RoughEstimationPW(FLOAT **Y, FLOAT *h, FLOAT nl, int m, ComponentDistributionType *RigidTheta, ComponentDistributionType *LooseTheta);
-    virtual int RoughEstimationH(int k, FLOAT **Y, FLOAT *h, FLOAT nl, int m, ComponentDistributionType *RigidTheta, ComponentDistributionType *LooseTheta);
-    virtual int ComponentDist(FLOAT *Y, ComponentDistributionType *CmpTheta, FLOAT *CmpDist, int Cumulative);
-    virtual int EnhancedEstimationKNN(FLOAT **Y, FLOAT nl, ComponentDistributionType *RigidTheta, ComponentDistributionType *LooseTheta);
-    virtual int EnhancedEstimationPW(FLOAT **Y, FLOAT nl, ComponentDistributionType *RigidTheta, ComponentDistributionType *LooseTheta);
-    virtual int EnhancedEstimationH(int k, FLOAT **Y, FLOAT nl, ComponentDistributionType *RigidTheta, ComponentDistributionType *LooseTheta);
-    virtual int MomentsCalculation(ComponentDistributionType *CmpTheta, FLOAT *FirstM, FLOAT *SecondM);
-    virtual int BayesClassificationKNN(FLOAT **Y, int c, FLOAT *W, ComponentDistributionType *Theta, FLOAT **FirstM, FLOAT **SecondM);
-    virtual int BayesClassificationPW(FLOAT **Y, int c, FLOAT *W, ComponentDistributionType *CmpTheta, FLOAT **FirstM, FLOAT **SecondM);
-    virtual int BayesClassificationH(int k, FLOAT **Y, int c, FLOAT *W, ComponentDistributionType *CmpTheta, FLOAT **FirstM, FLOAT **SecondM);
-    virtual int DegreesOffreedom(int c, ComponentDistributionType *CmpTheta, int *M);
-    int MixtureDist(FLOAT *Y, int c, FLOAT *W, ComponentDistributionType *CmpTheta, FLOAT *MixDist, int Cumulative);
-    int InformationCriterionKNN(int k, FLOAT **Y, int c, FLOAT *W, ComponentDistributionType *CmpTheta, FLOAT *IC, FLOAT *logL, int *M, FLOAT *D);
-    int InformationCriterionPW(FLOAT V, FLOAT **Y, int c, FLOAT *W, ComponentDistributionType *CmpTheta, FLOAT *IC, FLOAT *logL, int *M, FLOAT *D);
-    int InformationCriterionH(FLOAT V, int k, FLOAT **Y, int c, FLOAT *W, ComponentDistributionType *CmpTheta, FLOAT *IC, FLOAT *logL, int *M, FLOAT *D);
+    virtual int RoughEstimationKNN(FLOAT **Y, int k, FLOAT *h, FLOAT nl, int m, CompnentDistribution *RigidTheta, CompnentDistribution *LooseTheta);
+    virtual int RoughEstimationPW(FLOAT **Y, FLOAT *h, FLOAT nl, int m, CompnentDistribution *RigidTheta, CompnentDistribution *LooseTheta);
+    virtual int RoughEstimationH(int k, FLOAT **Y, FLOAT *h, FLOAT nl, int m, CompnentDistribution *RigidTheta, CompnentDistribution *LooseTheta);
+    virtual int ComponentDist(FLOAT *Y, CompnentDistribution *CmpTheta, FLOAT *CmpDist);
+    virtual int EnhancedEstimationKNN(FLOAT **Y, FLOAT nl, CompnentDistribution *RigidTheta, CompnentDistribution *LooseTheta);
+    virtual int EnhancedEstimationPW(FLOAT **Y, FLOAT nl, CompnentDistribution *RigidTheta, CompnentDistribution *LooseTheta);
+    virtual int EnhancedEstimationH(int k, FLOAT **Y, FLOAT nl, CompnentDistribution *RigidTheta, CompnentDistribution *LooseTheta);
+    virtual int MomentsCalculation(CompnentDistribution *CmpTheta, FLOAT *FirstM, FLOAT *SecondM);
+    virtual int BayesClassificationKNN(FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT **FirstM, FLOAT **SecondM);
+    virtual int BayesClassificationPW(FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT **FirstM, FLOAT **SecondM);
+    virtual int BayesClassificationH(int k, FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT **FirstM, FLOAT **SecondM);
+    virtual int DegreesOffreedom(int c, CompnentDistribution **MixTheta, int *M);
+    int MixtureDist(FLOAT *Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT *MixDist);
+    int InformationCriterionKNN(int k, FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT *IC, FLOAT *logL, int *M, FLOAT *D);
+    int InformationCriterionPW(FLOAT V, FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT *IC, FLOAT *logL, int *M, FLOAT *D);
+    int InformationCriterionH(FLOAT V, int k, FLOAT **Y, int c, FLOAT *W, CompnentDistribution **MixTheta, FLOAT *IC, FLOAT *logL, int *M, FLOAT *D);
     int REBMIX();
     int RunTemplateFile(char *file);
 }; // Rebmix
